package fr.pandacube.java.logviewer.test;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.nio.file.FileSystemException;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.util.Random;
import java.util.concurrent.atomic.AtomicBoolean;

import fr.pandacube.java.logviewer.Main;

/**
 * Test du logiciel LogViewer.<br/>
 * 
 * Lance un Thread qui écrit aléatoirement dans un fichier de la même manière que dans un fichier log.
 * Ce fichier est réinitialisé pour simuler la rotation des fichiers logs.
 * 
 * 
 * @author Marc
 *
 */
public class Test {
	
	
	private static BufferedWriter wr;
	
	public static void main(String[] args) throws Exception {
		
		File f = new File("testFile");
		
		
		f.createNewFile();
		
		AtomicBoolean terminate = new AtomicBoolean(false);
		
		wr = new BufferedWriter(new FileWriter(f));
		
		Thread th = new Thread(() -> {
			
			try {
				
				while(!terminate.get()) {
					
					wr.write(randomLine());
					wr.flush();
					Thread.sleep(r.nextInt(50));
					
					if (r.nextInt(100) == 0) {
						wr.close();
						
						// tant que l'exeption est lancée, on réessaye
						for(;;) {
							try {
								Files.move(f.toPath(), new File(r.nextInt(10)+".old").toPath(), StandardCopyOption.ATOMIC_MOVE);
								break;
							} catch (FileSystemException e) {
								Thread.sleep(250);
							}
						}
						
						f.createNewFile();
						wr = new BufferedWriter(new FileWriter(f));
					}
					
					
					
				}
				
			} catch (Exception e) {
				e.printStackTrace();
			}
			
		}, "Testing Thread");
		
		th.start();
		
		Thread.currentThread().setName("Tested Thread");
		Main.main(new String[] {"testFile"});
		
		terminate.set(true);
		th.join();
		
		wr.close();
		
	}
	
	
	
	
	
	
	
	
	
	private static Random r = new Random();
	private static char[] chars = new char[] {' ', ' ', ' ', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'};
	
	public static String randomLine() {
		int length = r.nextInt(100)+15;
		String s = "";
		for (int i=0; i<length; i++)
			s+=chars[r.nextInt(chars.length)];
		
		return s+'\n';
	}
	
	
	
	
	
}
